import javax.microedition.lcdui.*;
import java.util.*;

public class NewTimerForm extends Form implements CommandListener  
{ 
    Scheduler scheduler;
    Schedule  timer;
    Displayable parent;

    TextField name;
    TextField hour;
    TextField minute;
    TextField second;
    ChoiceGroup relative;
    ChoiceGroup periodic;

    static final int MAX_TIMER_NAME_LENGTH = 256;
    static final int DAY = 24*60*60;
    static final int RELATIVE = 0;
    static final int ABSOLUTE = 1;

    NewTimerForm(Scheduler scheduler, Displayable parent, Schedule schedule) { 
        super(Locale.SET_TIMER);
        this.scheduler = scheduler;
        this.parent = parent;
        if (schedule != null) { 
            addCommand(schedule.expiration != 0 ? Scheduler.STOP_CMD : Scheduler.START_CMD);
            timer = schedule;
        } else { 
            addCommand(Scheduler.START_CMD);
            schedule = new Schedule("", 0, 0, 0, true);
        }
        addCommand(Scheduler.SAVE_CMD);        
        addCommand(Scheduler.BACK_CMD);
        setCommandListener(this);

        name = new TextField(Locale.NAME, schedule.name, MAX_TIMER_NAME_LENGTH, TextField.ANY);
        hour = new TextField(Locale.HOUR, Integer.toString(schedule.fire/60/60), 8, TextField.NUMERIC);
        minute = new TextField(Locale.MINUTE, Integer.toString(schedule.fire/60%60), 8, TextField.NUMERIC);
        second = new TextField(Locale.SECOND, Integer.toString(schedule.fire%60), 8, TextField.NUMERIC);
        relative = new ChoiceGroup(null, Choice.EXCLUSIVE, new String[]{Locale.RELATIVE, Locale.ABSOLUTE}, null);
        periodic = new ChoiceGroup(null, Choice.MULTIPLE, new String[]{Locale.PERIODIC}, null);
        relative.setSelectedIndex(schedule.isRelative ? RELATIVE : ABSOLUTE, true);
        periodic.setSelectedFlags(new boolean[]{schedule.period != 0});
        append(name);
        append(hour);
        append(minute);
        append(second);
        append(relative);
        append(periodic);
        scheduler.display.setCurrent(this);
    }

    public void commandAction(Command c, Displayable d)  {
        if (c == Scheduler.START_CMD || c == Scheduler.SAVE_CMD) { 
            try {
                int fire, period;
                long curr = System.currentTimeMillis();
                long expiration;
                boolean isRelative = relative.getSelectedIndex() == RELATIVE;
                if (isRelative) { 
                    fire = period = (Integer.parseInt(hour.getString())*60 + Integer.parseInt(minute.getString()))*60 + Integer.parseInt(second.getString());
                    expiration = curr + (long)period*Scheduler.SECOND;
                } else { 
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(new Date(curr));
                    calendar.set(Calendar.HOUR_OF_DAY, Integer.parseInt(hour.getString()));
                    calendar.set(Calendar.MINUTE, Integer.parseInt(minute.getString()));
                    calendar.set(Calendar.SECOND, Integer.parseInt(second.getString()));
                    expiration = calendar.getTime().getTime();
                    calendar.set(Calendar.HOUR_OF_DAY, 0);
                    calendar.set(Calendar.MINUTE, 0);
                    calendar.set(Calendar.SECOND, 0);
                    fire = (int)((expiration - calendar.getTime().getTime())/Scheduler.SECOND);
                    period = DAY;
                }
                if (periodic.getSelectedFlags(new boolean[1]) == 0) { 
                    period = 0;
                }
                String timerName = name.getString().trim();
                if (timerName.length() == 0) { 
                    if (isRelative) {
                        timerName = (fire < 60) 
                          ? fire + " " + Locale.SEC
                          : (fire < 60*60 && fire % 60 == 0)
                            ? (fire/60) + " " + Locale.MIN
                            : (fire % (60*60) == 0)
                              ? (fire/60/60) + " " + Locale.H
                              : hour.getString() + ':' + minute.getString() + ':' + second.getString();
                    } else { 
                        timerName = Locale.ALARM;
                    }
                }
                timer = new Schedule(timerName, expiration, fire, period, isRelative);
                if (c == Scheduler.START_CMD) { 
                    scheduler.addTimer(timer, true);                
                    scheduler.display.setCurrent(scheduler.monitor);
                } else {
                    scheduler.addTimer(timer, false);                
                    scheduler.display.setCurrent(parent);
                }
            } catch (Exception x) { 
                x.printStackTrace();
                scheduler.error(Locale.INVALID_TIME, this);
            }
        } else if (c == Scheduler.STOP_CMD) { 
            removeCommand(Scheduler.STOP_CMD);
            addCommand(Scheduler.START_CMD);
            scheduler.stopTimer(timer);
        } else if (c == Scheduler.BACK_CMD) {
            scheduler.display.setCurrent(parent);
        }
    }
}        
        