import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import javax.wireless.messaging.*;
import javax.microedition.media.*;
import javax.microedition.io.*;
import java.io.*;
import java.util.*;

public class Scheduler extends MIDlet { 
    static Command QUIT_CMD = new Command(Locale.EXIT, Command.EXIT, 5);
    static Command BACK_CMD = new Command(Locale.BACK, Command.BACK, 5);
    static Command START_CMD = new Command(Locale.START, Command.OK, 2);
    static Command STOP_CMD = new Command(Locale.STOP, Command.OK, 2);
    static Command LIST_CMD = new Command(Locale.LIST, Command.OK, 3);
    static Command SAVE_CMD = new Command(Locale.SAVE, Command.OK, 3);
    static Command ADD_CMD = new Command(Locale.ADD, Command.OK, 2);
    static Command NEW_CMD = new Command(Locale.NEW, Command.OK, 2);
    static Command DELETE_CMD = new Command(Locale.DELETE, Command.ITEM, 4);
    static Command SETTINGS_CMD = new Command(Locale.SETTINGS, Command.OK, 3);
    static Command DISMISS_CMD = new Command(Locale.DISMISS, Command.OK, 1);

    static final String STORAGE_NAME = "TIMER";

    static final int SECOND = 1000;

    Vector      timers;
    RecordStore store;
    Monitor     monitor;
    Settings    settings;
    long        nextAlarm;
    boolean     alarmSound;
    Object      alarmMutex;
    Display     display;
    Displayable current;

    class Action extends TimerTask {
        Schedule schedule;

        Action(Schedule schedule) { 
            this.schedule = schedule;
        }

        public void run() { 
            alarm(schedule);
        }
    }

    void sendSms(final String message, final String destination) { 
        new Thread() {
            public void run() { 
                String addr = "sms://" + destination;
                MessageConnection smsconn = null;
                try {
                    smsconn = (MessageConnection) Connector.open(addr);
                    TextMessage msg =
                        (TextMessage)smsconn.newMessage(MessageConnection.TEXT_MESSAGE);
                    msg.setPayloadText(message);
                    smsconn.send(msg);
                } catch (Throwable x) {
                    x.printStackTrace();
                }
                if (smsconn != null) {
                    try {
                        smsconn.close();
                    } catch (IOException x) {
                        x.printStackTrace();
                    }                
                }
            }
        }.start();
    }
         
    void error(String message, Displayable parent) { 
        Alert alert = new Alert(Locale.ERROR, message, null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        display.setCurrent(alert, parent);
    }

    void turnOff() { 
        if ((settings.notification & Settings.BACKLIGHT) != 0) { 
            display.flashBacklight(0);
        }
        /*
        if ((settings.notification & Settings.VIBRA) != 0) { 
            display.vibrate(0);
        } 
        */
        synchronized(alarmMutex) { 
            alarmSound = false;
        }
        display.setCurrent(current);
    }

    class AlarmThread extends Thread {
        public void run() { 
            try { 
                int period = SECOND/settings.frequency;
                int repeat = settings.duration*settings.frequency; 
                while (--repeat >= 0) { 
                    synchronized(alarmMutex) { 
                        if (!alarmSound) { 
                            break;
                        }
                    }
                    Manager.playTone(settings.note, period >> 1, settings.volume);
                    if ((settings.notification & Settings.VIBRA) != 0) { 
                        display.vibrate(period >> 1);
                    }
                    sleep(period);
                }
            } catch (Exception x) {
                x.printStackTrace();
            }
            synchronized(alarmMutex) { 
                alarmSound = false;
            }
        }
    }
   
    void alarm(Schedule s) { 
        if ((settings.notification & Settings.BACKLIGHT) != 0) { 
            display.flashBacklight(settings.duration*SECOND);
        }
        if ((settings.notification & Settings.SOUND) != 0 && settings.frequency != 0) { 
            synchronized(alarmMutex) { 
                if (!alarmSound) { 
                    alarmSound = true;
                    new AlarmThread().start();
                }
            }
        } else if ((settings.notification & Settings.VIBRA) != 0) { 
            display.vibrate(settings.duration*SECOND);
        } 
        if ((settings.notification & Settings.SMS) != 0) { 
            sendSms(s.name, settings.sms);
        }
        if (settings.notification != Settings.SMS) {         
            Displayable curr = display.getCurrent();
            if (!(curr instanceof Alert)) {
                Alert alert = new Alert(Locale.ALARM, s.name, null, AlertType.ALARM);
                // alert.setTimeout(settings.duration*SECOND);
                alert.setTimeout(Alert.FOREVER);
                alert.setCommandListener(monitor);
                alert.addCommand(DISMISS_CMD);
                if (curr != null && curr.isShown()) { 
                    current = curr;
                } else if (current == null) { 
                    current = monitor;
                }
                display.setCurrent(alert);
            }
        }
        if (s.period != 0) {
            s.expiration += s.period*SECOND;
            updateTimer(s);
        } else {
            stopTimer(s);
        }
    }

    void updateTimer(Schedule s)
    {
        if (store != null) { 
            byte[] record = s.pack();
            try { 
                store.setRecord(s.id, record, 0, record.length);
            } catch(RecordStoreException x) { 
                x.printStackTrace();
            }
        }      
        reset();
    }

    void stopTimer(Schedule s)
    {
        if (s.timer != null) {
            s.timer.cancel();
            s.timer = null;
        }
        s.expiration = 0;
        updateTimer(s);
    }
    
    void reset() { 
        nextAlarm = Long.MAX_VALUE;
        for (int i = 0, n = timers.size(); i < n; i++) { 
            Schedule t = (Schedule)timers.elementAt(i);
            if (t.expiration != 0 && t.expiration < nextAlarm) { 
                nextAlarm = t.expiration;
            }
        }
        if (nextAlarm != Long.MAX_VALUE) {
            pushRegistry();
        }
        monitor.refresh();
    }

    void pushRegistry() {
        new Thread() { 
            public void run() { 
                try { 
                    PushRegistry.registerAlarm("Scheduler", nextAlarm);
                } catch (Exception x) {
                    x.printStackTrace();
                }
           } 
        }.start();
    }
    
    void addTimer(Schedule s, boolean start) { 
        for (int i = 0, n = timers.size(); i < n; i++) { 
            Schedule t = (Schedule)timers.elementAt(i);
            if (t.name.equals(s.name)) { 
                if (t.expiration != 0 || start) { 
                    t.expiration = s.expiration;
                }
                t.fire = s.fire;
                t.period = s.period;
                t.isRelative = s.isRelative;
                byte[] record = t.pack();
                if (store != null) { 
                    try { 
                        store.setRecord(t.id, record, 0, record.length);
                    } catch(RecordStoreException x) { 
                        x.printStackTrace();
                    }
                }
                s = t;
                break;
            }
        }
        if (s.id == 0) { 
            byte[] record = s.pack();
            if (store != null) { 
                try { 
                    s.id = store.addRecord(record, 0, record.length);
                } catch(RecordStoreException x) { 
                    x.printStackTrace();
                }
            }
            if (!start) { 
                s.expiration = 0;
            }
            timers.addElement(s);
        }
        if (s.expiration != 0) { 
            if (s.expiration < nextAlarm) { 
                nextAlarm = s.expiration;
                pushRegistry();
            }
            if (s.timer != null) { 
                s.timer.cancel();
                s.timer = null;
            }
            startTimer(s);
        }
        monitor.refresh();
    }

    void startTimer(Schedule t) {
        t.timer = new Timer();
        if (t.period != 0) { 
            t.timer.scheduleAtFixedRate(new Action(t), new Date(t.expiration), t.period*SECOND);
        } else { 
            t.timer.schedule(new Action(t), new Date(t.expiration));
        }
    }

    void deleteTimer(int i) {
        Schedule s = (Schedule)timers.elementAt(i);
        if (s.timer != null) {
            s.timer.cancel();
            s.timer = null;
        }
        if (store != null) { 
            try { 
                store.deleteRecord(s.id);
            } catch(RecordStoreException x) { 
                x.printStackTrace();
            }
        }
        timers.removeElementAt(i);
        reset();
    }
         
    public Scheduler() { 
        alarmMutex = new Object();
        timers = new Vector();
        settings = new Settings();
        display = Display.getDisplay(this);
        monitor = new Monitor(this);        
    }

    void saveSettings() { 
        if (store != null) { 
            byte[] record = settings.pack();
            try { 
                store.setRecord(1, record, 0, record.length);
            } catch(RecordStoreException x) { 
                x.printStackTrace();
            }
        }
    }

    private void openStore() {
        try { 
            store = RecordStore.openRecordStore(STORAGE_NAME, true);
            int nextId = store.getNextRecordID();
            if (nextId == 1) {
                byte[] record = settings.pack();
                store.addRecord(record, 0, record.length);
            } else { 
                settings = new Settings(store.getRecord(1));
                for (int i = 2; i < nextId; i++) { 
                    try { 
                        Schedule s = new Schedule(i, store.getRecord(i));
                        timers.addElement(s);
                    } catch (InvalidRecordIDException x) {
                    }
                }
            }
        } catch(RecordStoreException x) { 
            x.printStackTrace();
            store = null;
        }
    }

    private void closeStore() 
    {
        if (store != null) { 
            try { 
                store.closeRecordStore();
                store = null;
            } catch(RecordStoreException x) { 
                x.printStackTrace();
            }
        }
    }

    protected void destroyApp(boolean unconditional) 
    {
        closeStore();
    }  

    protected  void pauseApp() 
    {
    }

    protected void startApp() 
    {
        openStore();
        reset();
        long now = System.currentTimeMillis();
        for (int i = 0, n = timers.size(); i < n; i++) {
            Schedule s = (Schedule)timers.elementAt(i);
            if (s.expiration != 0) { 
                if (s.expiration < now) { 
                    alarm(s);
                } else {
                    startTimer(s);
                }
            }                                        
        }
    }

    void quit() { 
        destroyApp(true);
        notifyDestroyed();
    }
}
