import javax.microedition.lcdui.*;

public class SettingsForm extends Form implements CommandListener { 
    Scheduler   scheduler;
    Displayable parent;
    Gauge       note;
    Gauge       duration;
    Gauge       frequency;
    Gauge       volume;
    ChoiceGroup notification;
    TextField   sms;

    static final int MAX_NOTE = 127;
    static final int MAX_DURATION = 100;
    static final int MAX_FREQUENCY = 100;
    static final int MAX_VOLUME = 100;

    static final int MAX_SMS_RECIPIENT_LENGTH = 64;

    SettingsForm(Scheduler scheduler, Displayable parent) { 
        super(Locale.SETTINGS);        
        this.scheduler = scheduler;
        this.parent = parent;
        note = new Gauge(Locale.NOTE, true, MAX_NOTE, scheduler.settings.note);
        duration = new Gauge(Locale.DURATION, true, MAX_DURATION, scheduler.settings.duration);
        frequency = new Gauge(Locale.FREQUENCY, true, MAX_FREQUENCY, scheduler.settings.frequency);
        volume = new Gauge(Locale.VOLUME, true, MAX_VOLUME, scheduler.settings.volume);
        notification = new ChoiceGroup(Locale.NOTIFICATION, Choice.MULTIPLE, new String[] { Locale.SOUND, Locale.VIBRA, Locale.BACKLIGHT, Locale.SMS }, null);
        boolean[] selectedNotification = new boolean[4];
        if ((scheduler.settings.notification & Settings.SOUND) != 0) {
            selectedNotification[0] = true;
        }
        if ((scheduler.settings.notification & Settings.VIBRA) != 0) {
            selectedNotification[1] = true;
        }
        if ((scheduler.settings.notification & Settings.BACKLIGHT) != 0) {
            selectedNotification[2] = true;
        }
        if ((scheduler.settings.notification & Settings.SMS) != 0) {
            selectedNotification[3] = true;
        }
        notification.setSelectedFlags(selectedNotification);
        sms = new TextField(Locale.SMS_RECIPIENT, scheduler.settings.sms, MAX_SMS_RECIPIENT_LENGTH, TextField.PHONENUMBER); 
        append(note);
        append(duration);
        append(frequency);
        append(volume);
        append(notification);
        append(sms);
        addCommand(Scheduler.SAVE_CMD);
        addCommand(Scheduler.BACK_CMD);
        setCommandListener(this);
        scheduler.display.setCurrent(this);
    }
    
    public void commandAction(Command c, Displayable d) 
    {
        if (c == Scheduler.SAVE_CMD) { 
            boolean[] selectedNotification = new boolean[4];
            notification.getSelectedFlags(selectedNotification);
            scheduler.settings.notification = 0;
            if (selectedNotification[0]) {
                scheduler.settings.notification |= Settings.SOUND;
            } 
            if (selectedNotification[1]) {
                scheduler.settings.notification |= Settings.VIBRA;
            } 
            if (selectedNotification[2]) { 
                scheduler.settings.notification |= Settings.BACKLIGHT;
            }                 
            if (selectedNotification[3]) { 
                scheduler.settings.notification |= Settings.SMS;
            }                 
            scheduler.settings.note = note.getValue();
            scheduler.settings.duration = duration.getValue();
            scheduler.settings.frequency = frequency.getValue();
            scheduler.settings.volume = volume.getValue();
            scheduler.settings.sms = sms.getString();
            scheduler.saveSettings();
        }                
        scheduler.display.setCurrent(parent);
    }
}
           

   